@@ -5,6 +5,7 @@ grid.
 .. codeauthor:: David Zwicker <david.zwicker@ds.mpg.de>
 '''
 
+import json
 from typing import (Sequence, Optional, Union, Any, Dict,
                     List, Iterator)  # @UnusedImport
 
@@ -174,48 +175,56 @@ class FieldCollection(FieldBase):
     
         
     @classmethod
-    def from_state(cls, state: Dict[str, Any], grid: GridBase,  # type: ignore
-                   data=None) -> "FieldCollection":
+    def from_state(cls, attributes: Dict[str, Any],
+                   data: np.ndarray = None) -> "FieldCollection":
         """ create a field collection from given state.
         
         Args:
-            state (str or dict): State from which the instance is created. If 
-                `state` is a string, it is decoded as JSON.
-            grid (:class:`~pde.grids.GridBase`):
-                The grid that is used to describe the field
-            data (:class:`numpy.ndarray`, optional): Data values at the support
-                points of the grid that define the field.
+            attributes (dict):
+                The attributes that describe the current instance
+            data (:class:`numpy.ndarray`, optional):
+                Data values at support points of the grid defining all fields
         """
-        fields = [FieldBase.from_state(field_state, grid=grid)
-                  for field_state in state.pop('fields')]
+        if 'class' in attributes:
+            class_name = attributes.pop('class')
+            assert class_name == cls.__name__
         
-        return cls(fields, data=data, **state)  # type: ignore
+        # restore the individual fields (without data)
+        fields = [FieldBase.from_state(field_state)
+                  for field_state in attributes.pop('fields')]
+        
+        return cls(fields, data=data, **attributes)  # type: ignore
 
 
     @classmethod
     def _from_hdf_dataset(cls, dataset) -> "FieldCollection":
         """ construct the class by reading data from an hdf5 dataset """
-        count = dataset.attrs['count']  # number of fields
-        label = dataset.attrs['label'] if 'label' in dataset.attrs else None
+        # copy attributes from hdf
+        attributes = dict(dataset.attrs)
         
-        # reconstruct all fields
-        fields = [FieldBase._from_hdf_dataset(dataset[f"field_{i}"])
-                  for i in range(count)]
-             
-        return cls(fields, label=label)  # type: ignore
+        # determine class
+        class_name = json.loads(attributes.pop('class'))
+        assert class_name == cls.__name__
+        
+        # determine the fields
+        field_attrs = json.loads(attributes.pop('fields'))
+        fields = [DataFieldBase._from_hdf_dataset(dataset[f"field_{i}"])
+                  for i in range(len(field_attrs))]
+        
+        # unserialize remaining attributes
+        attributes = cls.unserialize_attributes(attributes)
+        return cls(fields, **attributes)  # type: ignore
 
 
-    def _write_hdf_dataset(self, fp):
-        """ write data to a given hdf5 file pointer `fp` """
-        # write details of the collection
-        fp.attrs['class'] = self.__class__.__name__
-        fp.attrs['count'] = len(self)
-        if self.label:      
-            fp.attrs['label'] = str(self.label)
+    def _write_hdf_dataset(self, hdf_path):
+        """ write data to a given hdf5 path `hdf_path` """
+        # write attributes of the collection
+        for key, value in self.attributes_serialized.items():
+            hdf_path.attrs[key] = value
                   
         # write individual fields
         for i, field in enumerate(self.fields):
-            field._write_hdf_dataset(fp, f"field_{i}")
+            field._write_hdf_dataset(hdf_path, f"field_{i}")
 
 
     def assert_field_compatible(self, other: FieldBase,
@@ -255,26 +264,47 @@ class FieldCollection(FieldBase):
     
     
     @property
-    def state(self) -> dict:
-        """ dict: current state of this instance """
-        return {'label': self.label,
-                'fields': [f.state for f in self.fields]}
-
+    def attributes(self) -> Dict[str, Any]:
+        """ dict: describes the state of the instance (without the data) """
+        results = super().attributes
+        del results['grid']
+        results['fields'] = [f.attributes for f in self.fields]
+        return results
+    
 
     @property
-    def state_serialized(self) -> str:
-        """ str: a json serialized version of the field """
-        import json
+    def attributes_serialized(self) -> Dict[str, str]:
+        """ dict: serialized version of the attributes """
+        results = {}
+        for key, value in self.attributes.items():
+            if key == 'fields':
+                fields = [f.attributes_serialized for f in self.fields]
+                results[key] = json.dumps(fields)
+            else:
+                results[key] = json.dumps(value)
+        return results
+    
+    
+    @classmethod
+    def unserialize_attributes(cls, attributes: Dict[str, str]) \
+            -> Dict[str, Any]:
+        """ unserializes the given attributes
         
-        fields = []
-        for field in self.fields:
-            state = field.state
-            state['class'] = field.__class__.__name__
-            fields.append(state)
-
-        return json.dumps({'class': self.__class__.__name__,
-                           'label': self.label,
-                           'fields': fields})
+        Args:
+            attributes (dict):
+                The serialized attributes
+                
+        Returns:
+            dict: The unserialized attributes
+        """
+        results = {}
+        for key, value in attributes.items():
+            if key == 'fields':
+                results[key] = [FieldBase.unserialize_attributes(attrs)
+                                for attrs in json.loads(value)]
+            else:
+                results[key] = json.loads(value)
+        return results
     
     
     def copy(self, data=None, label: str = None) -> 'FieldCollection':
@@ -388,98 +418,151 @@ class FieldCollection(FieldBase):
     
     def plot_collection(self,
                         title: Optional[str] = None,
+                        tight: bool = True,
+                        filename: str = None,
                         show: bool = False,
                         **kwargs):
         r""" visualize all fields by plotting them next to each other
         
         Args:
-            title (str): Title of the plot. If omitted, the title is chosen
-                automatically based on the label the data field.
+            title (str):
+                Title of the plot. If omitted, the title is chosen automatically
+                based on the label the data field.
+            tight (bool):
+                Whether to call :func:`matplotlib.pyploy.tight_layout`
+            filename (str, optional):
+                If given, the plot is written to the specified file.
             show (bool):
-                Flag setting whether :func:`matplotlib.pyplot.show` is called
-            \**kwargs: Additional keyword arguments are passed to the method
-                `plot_line` of the individual fields.
+                Whether to call :func:`matplotlib.pyplot.show`
+            \**kwargs:
+                Additional keyword arguments are passed to the method
+                plotting function of the individual fields.
+                
+        Returns:
+            :class:`matplotlib.figure.Figure`: The figure instance
         """
         import matplotlib.pyplot as plt
-        _, axes = plt.subplots(1, len(self), figsize=(4 * len(self), 3))
+        
+        # plot the individual panels
+        fig, axes = plt.subplots(1, len(self), figsize=(4 * len(self), 3))
         for field, ax in zip(self.fields, axes):
             field.plot(ax=ax, **kwargs)
-        if title is None:
-            title = self.label
-        plt.suptitle(title)
-        if show:
-            plt.show()
+            
+        # set the title above all panels
+        fig.suptitle(self.label if title is None else title)
+            
+        # adjust layout
+        if tight:
+            fig.tight_layout()
+            
+        from ..visualization.plotting import finalize_plot
+        finalize_plot(fig, filename=filename, show=show)
+        return fig
     
     
-    def plot_line(self,
-                  title: Optional[str] = None,
-                  show: bool = False,
-                  **kwargs):
-        r""" visualize all fields using a 1d cuts
+    def plot_line(self, **kwargs):
+        r""" visualize all fields using line plots
         
         Args:
-            title (str): Title of the plot. If omitted, the title is chosen
-                automatically based on the label the data field.
-            show (bool):
-                Flag setting whether :func:`matplotlib.pyplot.show` is called
-            \**kwargs: Additional keyword arguments are passed to the method
-                `plot_line` of the individual fields.
+            \**kwargs:
+                Supported keyword arguments include `title` to set the title,
+                `tight` to adjust the layout, `filename` to write the image to a
+                file, and `show` to call :func:`matplotlib.pyplot.show`. These
+                are described in :meth:`~FieldCollection.plot_collection`.
+                
+        Returns:
+            :class:`matplotlib.figure.Figure`: The figure instance
         """
-        self.plot_collection(title=title, show=show, kind='line', **kwargs)
+        return self.plot_collection(kind='line', **kwargs)
         
+        
+    def plot_vector(self, **kwargs):
+        r""" visualize all fields using vector plots
+        
+        Args:
+            \**kwargs:
+                Supported keyword arguments include `title` to set the title,
+                `tight` to adjust the layout, `filename` to write the image to a
+                file, and `show` to call :func:`matplotlib.pyplot.show`. These
+                are described in :meth:`~FieldCollection.plot_collection`.
+                
+        Returns:
+            :class:`matplotlib.figure.Figure`: The figure instance
+        """
+        return self.plot_collection(kind='vector', **kwargs)
+                
     
-    def plot_image(self, quantities=None,  # type: ignore
+    def plot_image(self, quantities=None,
                    title: Optional[str] = None,
-                   show: bool = False,
+                   tight: bool = True,
+                   filename: str = None,
+                   show: bool = True,
                    **kwargs):
-        r""" visualize images of all fields
+        r""" visualize all fields using density plots
         
         Args:
-            quantities: Determines what exactly is plotted. See
-                :class:`~pde.visualization.plotting.ScalarfieldPlot` for
-                details.
-            title (str): Title of the plot. If omitted, the title is chosen
-                automatically based on the label the data field.
+            quantities:
+                Determines what exactly is plotted. See
+                :class:`~pde.visualization.plotting.ScalarfieldPlot` for details
+            title (str):
+                Title of the plot. If omitted, the title is chosen automatically
+                based on the label the data field.
+            tight (bool):
+                Whether to call :func:`matplotlib.pyploy.tight_layout`
+            filename (str, optional):
+                If given, the plot is written to the specified file.
             show (bool):
                 Flag setting whether :func:`matplotlib.pyplot.show` is called
-            \**kwargs: Additional keyword arguments are passed to the plot
-                methods of the individual fields
+            \**kwargs:
+                Additional keyword arguments are passed to the plot methods of
+                the individual fields
+                
+        Returns:
+            :class:`matplotlib.figure.Figure`: The figure instance
         """
-        from ..visualization.plotting import ScalarFieldPlot
-        plot = ScalarFieldPlot(self, quantities=quantities, show=show)
         if title is None:
             title = self.label
-        plot.show_data(self, title=title)
+            
+        # create the plot panels
+        from ..visualization.plotting import ScalarFieldPlot
+        plot = ScalarFieldPlot(self, quantities=quantities, title=title,
+                               tight=tight, show=show)
+        
+        # save plot to file if requested
+        if filename:
+            plot.fig.savefig(filename)
+            
+        return plot.fig
         
         
-    def plot(self, kind: str = 'auto', filename=None, **kwargs):
-        r""" visualize the field
+    def plot(self, kind: str = 'auto', **kwargs):
+        r""" visualize the field collection
         
         Args:
-            kind (str): Determines the visualizations. Supported values are
+            kind (str):
+                Determines how to visualize the collection. Supported values are
                 `image`,  `line`, or `vector`. Alternatively, `auto` determines
-                the best visualization based on the field itself.
-            filename (str, optional): If given, the plot is written to the
-                specified file. Otherwise, the plot might show directly in an
-                interactive matplotlib session or `matplotlib.pyplot.show()`
-                might be used to display the graphics.
-            \**kwargs: All additional keyword arguments are forwarded to the
-                actual plotting functions.
+                the best visualization based on each individual field itself.
+            \**kwargs:
+                All additional keyword arguments are forwarded to the actual
+                plotting functions. They include the keyword arguments `title`
+                to set the title, `tight` to adjust the layout, `filename` to
+                write the image to a file, and `show` to call
+                :func:`matplotlib.pyplot.show`. These arguments are described in
+                :meth:`~FieldCollection.plot_collection`.
+                
+        Returns:
+            :class:`matplotlib.figure.Figure`: The figure instance
         """
         if kind == 'auto':
-            self.plot_collection(**kwargs)
+            return self.plot_collection(**kwargs)
         elif kind == 'image':
-            self.plot_image(**kwargs)
+            return self.plot_image(**kwargs)
         elif kind == 'line':
-            self.plot_line(**kwargs)
+            return self.plot_line(**kwargs)
         elif kind == 'vector':
-            self.plot_vector(**kwargs)
+            return self.plot_vector(**kwargs)
         else:
             raise ValueError(f'Unsupported plot `{kind}`. Possible choices are '
                              '`image`, `line`, `vector`, or `auto`.')
-            
-        # store the result to a file if requested
-        if filename:
-            import matplotlib.pyplot as plt
-            plt.savefig(filename)
     
