@@ -48,11 +48,13 @@ class VectorField(DataFieldBase):
         Note that the data of the scalar fields is copied in the process
         
         Args:
-            fields (list): The list of (compatible) scalar fields
-            label (str, optional): Name of the field
+            fields (list):
+                The list of (compatible) scalar fields
+            label (str, optional):
+                Name of the returned field
             
         Returns:
-            VectorField: the resulting vector field
+            :class:`VectorField`: the resulting vector field
         """
         grid = fields[0].grid
         
@@ -69,9 +71,13 @@ class VectorField(DataFieldBase):
     
 
     @classmethod
+    @fill_in_docstring
     def from_expression(cls, grid: GridBase, expressions: Sequence[str],
                         label: str = None) -> "VectorField":
         """ create a vector field on a grid from given expressions
+
+        Warning:
+            {WARNING_EXEC}
         
         Args:
             grid (:class:`~pde.grids.GridBase`):
@@ -90,17 +96,21 @@ class VectorField(DataFieldBase):
             raise ValueError(f'Expected {grid.dim} expressions for the '
                              f'coordinates {axes_names}.') 
         
-        # obtain the values at the grid points
+        # obtain the coordinates of the grid points
         points = {name: grid.cell_coords[..., i]
                   for i, name in enumerate(grid.axes)}
         
-        # evaluate the different components
+        # evaluate all vector components at all points 
         data = []
         for expression in expressions:
-            expr = ScalarExpression(expression=expression, signature=grid.axes) 
-            data.append(expr(**points))
+            expr = ScalarExpression(expression=expression, signature=grid.axes)
+            values = np.broadcast_to(expr(**points), grid.shape) 
+            data.append(values)
 
-        return cls(grid=grid, data=data, label=label)
+        # create vector field from the data
+        return cls(grid=grid,  # lgtm [py/call-to-non-callable]
+                   data=data,
+                   label=label)
         
 
     def __getitem__(self, key: int) -> ScalarField:
@@ -184,7 +194,8 @@ class VectorField(DataFieldBase):
             return out
         
         if nb.config.DISABLE_JIT:
-            def dot(a, b, out=None):
+            def dot(a: np.ndarray, b: np.ndarray, out: np.ndarray = None) \
+                    -> np.ndarray:
                 """ wrapper deciding whether the underlying function is called
                 with or without `out`. """
                 if out is None:
@@ -193,7 +204,8 @@ class VectorField(DataFieldBase):
             
         else:
             @nb.generated_jit
-            def dot(a, b, out=None):
+            def dot(a: np.ndarray, b: np.ndarray, out: np.ndarray = None) \
+                    -> np.ndarray:
                 """ wrapper deciding whether the underlying function is called
                 with or without `out`. """
                 if isinstance(a, nb.types.Number):
@@ -220,10 +232,12 @@ class VectorField(DataFieldBase):
         """ calculate the outer product of this vector field with another
         
         Args:
-            other (VectorField): The other term
-            out (Tensor2Field, optional): Optional tensorial field to which the 
-                result is written.
-            label (str, optional): Name of the returned field
+            other (:class:`VectorField`):
+                The second vector field
+            out (:class:`pde.fields.tensorial.Tensor2Field`, optional):
+                Optional tensorial field to which the  result is written.
+            label (str, optional):
+                Name of the returned field
         
         """
         from .tensorial import Tensor2Field  # @Reimport
@@ -250,6 +264,7 @@ class VectorField(DataFieldBase):
         
         Args:
             bc: 
+                The boundary conditions applied to the field.
                 {ARG_BOUNDARIES}
             out (ScalarField, optional):
                 Optional scalar field to which the  result is written.
@@ -277,6 +292,7 @@ class VectorField(DataFieldBase):
         
         Args:
             bc:
+                The boundary conditions applied to the field.
                 {ARG_BOUNDARIES}
             out (Tensor2Field, optional):
                 Optional tensorial field to which the  result is written.
@@ -306,6 +322,7 @@ class VectorField(DataFieldBase):
         
         Args:
             bc: 
+                The boundary conditions applied to the field.
                 {ARG_BOUNDARIES}
             out (VectorField, optional):
                 Optional vector field to which the  result is written.
@@ -334,10 +351,12 @@ class VectorField(DataFieldBase):
         The two tensor invariants are given by
         
         Args:
-            scalar (str or int): Choose the method to use. Possible 
-                choices are `norm`, `max`, `min`, `squared_sum`, and an integer
-                which specifies a particular component.
-            label (str, optional): Name of the returned field
+            scalar (str or int):
+                Choose the method to use. Possible  choices are `norm`, `max`,
+                `min`, `squared_sum`, and an integer which specifies a
+                particular component.
+            label (str, optional):
+                Name of the returned field
             
         Returns:
             ScalarField: the scalar result
@@ -367,8 +386,9 @@ class VectorField(DataFieldBase):
         """ return data for a line plot of the field
         
         Args:
-            method (str or int): The method for extracting scalars as described
-                in `self.to_scalar`.
+            method (str or int):
+                The method for extracting scalars as described in
+                `self.to_scalar`.
                 
         Returns:
             dict: Information useful for performing a line plot of the field
@@ -380,8 +400,9 @@ class VectorField(DataFieldBase):
         r""" return data for plotting an image of the field
 
         Args:
-            scalar (str or int): The method for extracting scalars as described
-                in `self.to_scalar`.
+            scalar (str or int):
+                The method for extracting scalars as described in
+                `self.to_scalar`.
             \**kwargs: Additional parameters are forwarded to `get_image_data`
                 
         Returns:
