@@ -36,6 +36,7 @@ from numba.extending import register_jitable
 
 from ..base import GridBase
 from ...tools.expressions import ScalarExpression
+from ...tools.docstrings import fill_in_docstring
 
 
 
@@ -168,12 +169,17 @@ class BCBase(metaclass=ABCMeta):
     homogeneous: bool  # spatial dependence of boundary condition?
     value: Any         # the actual value of the boundary condition
 
-    _subclasses: Dict[str, Any] = {}  # all classes inheriting from this
-    _conditions: Dict[str, Any] = {}
-    
+    _subclasses: Dict[str, 'BCBase'] = {}  # all classes inheriting from this
+    _conditions: Dict[str, 'BCBase'] = {}
     
+
+    @fill_in_docstring    
     def __init__(self, grid: GridBase, axis: int, upper: bool, value=0):
         """ 
+        Warning:
+            {WARNING_EXEC} However, the function is safe when `value` cannot be
+            an arbitrary string.
+        
         Args:
             grid (:class:`~pde.grids.GridBase`):
                 The grid for which the boundary conditions are defined
@@ -198,10 +204,14 @@ class BCBase(metaclass=ABCMeta):
         self.upper = upper
         self.set_value(value)
                 
-                    
+          
+    @fill_in_docstring          
     def set_value(self, value=0):
         """ set the value of this boundary condition
         
+        Warning:
+            {WARNING_EXEC}
+        
         Args:
             value (float or str or array):
                 a value stored with the boundary condition. The interpretation
