@@ -13,34 +13,30 @@ This module implements differential operators on spherical grids
    
    
 .. codeauthor:: David Zwicker <david.zwicker@ds.mpg.de>
-
-
-.. The following contains text parts that are used multiple times below:
-
-.. |Description_spherical| replace:: 
-  This function assumes spherical symmetry of the grid, so that fields only
-  depend on the radial coordinate `r`. The radial discretization is defined as
-  :math:`r_i = (i + \frac12) \Delta r` for :math:`i=0, \ldots, N_r-1`.
 """
 
 from typing import Callable
 
 import numpy as np
+from scipy import sparse
 
+from .common import make_poisson_solver
 from .. import SphericalGrid
 from ..boundaries import Boundaries
 from ...tools.numba import jit_allocate_out
+from ...tools.docstrings import fill_in_docstring
 
 
 
+@fill_in_docstring
 def make_laplace(bcs: Boundaries, conservative: bool = True) -> Callable:
     """ make a discretized laplace operator for a spherical grid
     
-    |Description_spherical|
+    {DESCR_SPHERICAL_GRID}
 
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         conservative (bool): flag indicating whether the laplace operator should
             be conservative (which results in slightly slower computations).    
         
@@ -54,7 +50,7 @@ def make_laplace(bcs: Boundaries, conservative: bool = True) -> Callable:
     dim_r = bcs.grid.shape[0]
     dr = bcs.grid.discretization[0]
     rs = bcs.grid.axes_coords[0]
-    r_min, _ = bcs.grid.axes_bounds[0]
+    r_min, r_max = bcs.grid.axes_bounds[0]
 
     # prepare boundary values
     value_lower_bc = bcs[0].low.get_virtual_point_evaluator()
@@ -64,6 +60,7 @@ def make_laplace(bcs: Boundaries, conservative: bool = True) -> Callable:
         # create a conservative spherical laplace operator
         rl = r_min + dr * np.arange(dim_r)  # inner radii of spherical shells
         rh = rl + dr                        # outer radii
+        assert np.isclose(rh[-1], r_max)
         volumes = (rh**3 - rl**3) / 3       # volume of the spherical shells
     
         if r_min == 0:
@@ -160,14 +157,15 @@ def make_laplace(bcs: Boundaries, conservative: bool = True) -> Callable:
 
 
 
+@fill_in_docstring
 def make_gradient(bcs: Boundaries) -> Callable:
     """ make a discretized gradient operator for a spherical grid
     
-    |Description_spherical|
+    {DESCR_SPHERICAL_GRID}
 
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values
@@ -214,14 +212,15 @@ def make_gradient(bcs: Boundaries) -> Callable:
 
     
 
+@fill_in_docstring
 def make_divergence(bcs: Boundaries) -> Callable:
     """ make a discretized divergence operator for a spherical grid
     
-    |Description_spherical|
+    {DESCR_SPHERICAL_GRID}
 
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values
@@ -286,14 +285,15 @@ def make_divergence(bcs: Boundaries) -> Callable:
 
 
     
+@fill_in_docstring
 def make_vector_gradient(bcs: Boundaries) -> Callable:
     """ make a discretized vector gradient operator for a spherical grid
     
-    |Description_spherical|
+    {DESCR_SPHERICAL_GRID}
 
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values
@@ -317,14 +317,15 @@ def make_vector_gradient(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
 def make_tensor_divergence(bcs: Boundaries) -> Callable:
     """ make a discretized tensor divergence operator for a spherical grid
     
-    |Description_spherical|
+    {DESCR_SPHERICAL_GRID}
 
     Args:
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that can be applied to an array of values
@@ -348,16 +349,79 @@ def make_tensor_divergence(bcs: Boundaries) -> Callable:
 
 
 
+@fill_in_docstring
+def _get_laplace_matrix(bcs):
+    """ get sparse matrix for laplace operator on a polar grid
+    
+    Args:
+        bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
+            {ARG_BOUNDARIES_INSTANCE}
+        
+    Returns:
+        tuple: A sparse matrix and a sparse vector that can be used to evaluate
+        the discretized laplacian
+    """    
+    assert isinstance(bcs.grid, SphericalGrid)
+    bcs.check_value_rank(0)
+
+    # calculate preliminary quantities
+    dim_r = bcs.grid.shape[0]
+    dr = bcs.grid.discretization[0]
+    rs = bcs.grid.axes_coords[0]
+    r_min, r_max = bcs.grid.axes_bounds[0]
+    
+
+    # create a conservative spherical laplace operator
+    rl = r_min + dr * np.arange(dim_r)  # inner radii of spherical shells
+    rh = rl + dr                        # outer radii
+    assert np.isclose(rh[-1], r_max)
+    volumes = (rh**3 - rl**3) / 3       # volume of the spherical shells
+
+    factor_l = (rs - 0.5 * dr)**2 / (dr * volumes)
+    factor_h = (rs + 0.5 * dr)**2 / (dr * volumes)
+
+    matrix = sparse.dok_matrix((dim_r, dim_r))
+    vector = sparse.dok_matrix((dim_r, 1))
+    
+    for i in range(dim_r):
+        matrix[i, i] += -factor_l[i] - factor_h[i]
+        
+        if i == 0:
+            if r_min == 0:
+                matrix[i, i + 1] = factor_l[i]
+            else:
+                const, entries = bcs[0].get_data((-1,))
+                vector[i] += const * factor_l[i]
+                for k, v in entries.items():
+                    matrix[i, k] += v * factor_l[i]
+
+        else:
+            matrix[i, i - 1] = factor_l[i]
+            
+        if i == dim_r - 1:
+            const, entries = bcs[0].get_data((dim_r,))
+            vector[i] += const * factor_h[i]
+            for k, v in entries.items():
+                matrix[i, k] += v * factor_h[i]
+                
+        else:
+            matrix[i, i + 1] = factor_h[i]
+            
+    return matrix, vector
+
+
+
+@fill_in_docstring
 def make_operator(op: str, bcs: Boundaries) -> Callable:
     """ make a discretized operator for a spherical grid
     
-    |Description_spherical|
+    {DESCR_SPHERICAL_GRID}
 
     Args:
         op (str): Identifier for the operator. Some examples are 'laplace',
             'gradient', or 'divergence'.
         bcs (:class:`~pde.grids.boundaries.axes.Boundaries`):
-            |Arg_boundary_conditions|
+            {ARG_BOUNDARIES_INSTANCE}
         
     Returns:
         A function that takes the discretized data as an input and returns
@@ -375,6 +439,8 @@ def make_operator(op: str, bcs: Boundaries) -> Callable:
         return make_vector_gradient(bcs)
     elif op == 'tensor_divergence':
         return make_tensor_divergence(bcs)
+    elif op == 'poisson_solver' or op == 'solve_poisson' or op == 'poisson':
+        return make_poisson_solver(*_get_laplace_matrix(bcs))
     else:
         raise NotImplementedError(f'Operator `{op}` is not defined for '
                                   'spherical grids')
