@@ -226,21 +226,28 @@ class SphericalGridBase(GridBase,  # lgtm [py/missing-equals]
 
 
     def get_image_data(self, data, performance_goal: str = 'speed',
-                       fill_value: float = 0) -> Dict[str, Any]:
+                       fill_value: float = 0,
+                       masked: bool = True) -> Dict[str, Any]:
         """ return a 2d-image of the data
         
         Args:
-            data (:class:`numpy.ndarray`): The values at the grid points
-            performance_goal (str): Determines the method chosen for
-                interpolation. Possible options are `speed` and `quality`.
-            fill_value (float): The value assigned to invalid positions (those
-                inside the hole or outside the bcs).
+            data (:class:`numpy.ndarray`):
+                The values at the grid points
+            performance_goal (str):
+                Determines the method chosen for interpolation. Possible options
+                are `speed` and `quality`.
+            fill_value (float):
+                The value assigned to invalid positions (those inside the hole
+                or outside the region).
+            masked (bool):
+                Whether a :class:`numpy.ma.MaskedArray` is returned for the data
+                instead of the normal :class:`numpy.ndarray`.
             
         Returns:
             A dictionary with information about the image, which is  convenient
             for plotting.
         """
-        r_outer = self.axes_bounds[0][1] 
+        _, r_outer = self.axes_bounds[0]
         r_data = self.axes_coords[0]
         
         if self.has_hole:
@@ -269,6 +276,10 @@ class SphericalGridBase(GridBase,  # lgtm [py/missing-equals]
         else:
             raise ValueError(f'Performance goal `{performance_goal}` undefined')
         
+        if masked:
+            mask = (r_img < r_data[0]) | (r_data[-1] < r_img) 
+            data_int = np.ma.masked_array(data_int, mask=mask)
+        
         return {'data': data_int,
                 'x': x, 'y': x,
                 'extent': (-r_outer, r_outer, -r_outer, r_outer),
@@ -426,6 +437,7 @@ class SphericalGridBase(GridBase,  # lgtm [py/missing-equals]
         
         Args:
             bc (str or list or tuple or dict):
+                The boundary conditions applied to the field.
                 {ARG_BOUNDARIES}
 
         Raises:
@@ -547,14 +559,19 @@ class PolarGrid(SphericalGridBase):
         return np.stack((x, y), axis=-1)
 
 
-    def plot(self, **kwargs):
-        r""" visualize the grid
+    def plot(self, title: str = None, show: bool = False, **kwargs):
+        r""" visualize the polar grid
         
         Args:
+            title (str):
+                Determines the title of the figure
+            show (bool):
+                Determines whether :func:`matplotlib.pyplot.show` is called
             \**kwargs: Extra arguments are passed on the to the matplotlib
                 plotting routines, e.g., to set the color of the lines
         """
         import matplotlib.pyplot as plt
+        from ..visualization.plotting import finalize_plot
         
         kwargs.setdefault('color', 'k')
         rb, = self.axes_bounds
@@ -571,6 +588,8 @@ class PolarGrid(SphericalGridBase):
         plt.ylim(-rmax, rmax)
         plt.ylabel('y')
         ax.set_aspect(1)
+        
+        finalize_plot(title=title, show=show)
             
     
     @cached_method()
@@ -582,6 +601,7 @@ class PolarGrid(SphericalGridBase):
             op (str): Identifier for the operator. Some examples are 'laplace',
                 'gradient', or 'divergence'.
             bc (str or list or tuple or dict):
+                The boundary conditions applied to the field.
                 {ARG_BOUNDARIES}
                 
         Returns:
@@ -649,6 +669,7 @@ class SphericalGrid(SphericalGridBase):
             op (str): Identifier for the operator. Some examples are 'laplace',
                 'gradient', or 'divergence'.
             bc (str or list or tuple or dict):
+                The boundary conditions applied to the field.
                 {ARG_BOUNDARIES}
                 
         Returns:
