@@ -49,20 +49,13 @@ def test_cylindrical_to_cartesian():
     expr_cart = expr_cyl.replace('r**2', '(x**2 + y**2)')
      
     z_range = (-np.pi, 2*np.pi)
-    grid_cyl = CylindricalGrid(8, z_range, (16, 33))
+    grid_cyl = CylindricalGrid(10, z_range, (16, 33))
     pf_cyl = ScalarField.from_expression(grid_cyl, expression=expr_cyl)
     
-    # pre-defined grid
     grid_cart = CartesianGrid([[-7, 7], [-6, 7], z_range], [16, 16, 16])
-    data_cart = grid_cyl.interpolate_to_cartesian(pf_cyl.data, grid=grid_cart)
-    pf_cart = ScalarField.from_expression(grid_cart, expression=expr_cart) 
-    np.testing.assert_allclose(data_cart, pf_cart.data, atol=0.1)
-     
-    # choose grid automatically
-    data_cart, grid_cart = grid_cyl.interpolate_to_cartesian(pf_cyl.data,
-                                                             ret_grid=True)
-    pf_cart = ScalarField.from_expression(grid_cart, expression=expr_cart) 
-    np.testing.assert_allclose(data_cart, pf_cart.data, atol=0.1)
+    pf_cart1 = pf_cyl.interpolate_to_grid(grid_cart)
+    pf_cart2 = ScalarField.from_expression(grid_cart, expression=expr_cart) 
+    np.testing.assert_allclose(pf_cart1.data, pf_cart2.data, atol=0.1)
 
 
 
@@ -85,3 +78,14 @@ def test_setting_domain_cylindrical():
         grid.get_boundary_conditions(['no-flux', 'no-flux'])
         
         
+
+@pytest.mark.parametrize('periodic', [True, False])
+def test_polar_conversion(periodic):
+    """ test conversion to polar coordinates """
+    grid = CylindricalGrid(1, [-1, 1], [5, 5], periodic_z=periodic)
+    dists = grid.polar_coordinates_real([0, 0, 0])
+    assert np.all(0.09 <= dists) 
+    assert np.any(dists < 0.11)
+    assert np.all(dists <= np.sqrt(2)) 
+    assert np.any(dists > 0.8 * np.sqrt(2)) 
+        
