@@ -12,15 +12,16 @@ Functions for creating movies of simulation results
 .. codeauthor:: David Zwicker <david.zwicker@ds.mpg.de>
 '''
 
-from typing import Union, Tuple
 import functools
 import os
 import subprocess as sp
 import tempfile
 import shutil
+from typing import Dict, Any
 
-from .plotting import ScalarFieldPlot
+from .plotting import ScalarFieldPlot, ScaleData
 from ..storage.base import StorageBase
+from ..tools.docstrings import fill_in_docstring
 
 
 
@@ -175,55 +176,63 @@ class Movie(object):
 
 
 
+@fill_in_docstring
 def movie_scalar(storage: StorageBase,
                  filename: str,
-                 scale: Union[str, float, Tuple[float, float]] = 'automatic',
-                 extras=None,
+                 scale: ScaleData = 'automatic',
+                 extras: Dict[str, Any] = None,
                  progress: bool = True,) -> None:
     """ produce a movie for a simulation of a scalar field
     
     Args:
         storage (:class:`~pde.storage.base.StorageBase`):
             The storage instance that contains all the data for the movie
-        filename (str): The filename to which the movie is written. The
-            extension determines the format used.
-        scale (float, tuple): Defining the color range shown; typically, two
-            numbers are given to define the lower and upper bound, but if only
-            one is specified the range [0, scale] is assumed. If `scale` is set
-            to 'automatic', the values are determined from the data in storage.
-        extras (dict, optional): Additional functions that are evaluated and
-            shown for each time step. The key of the dictionary is used as a
-            panel title.
-        progress (bool): Flag determining whether the progress of making
-            the movie is shown.
+        filename (str):
+            The filename to which the movie is written. The extension determines
+            the format used.
+        scale (str, float, tuple of float):
+            {ARG_PLOT_SCALE}
+        extras (dict, optional):
+            Additional functions that are evaluated and shown for each time 
+            step. The key of the dictionary is used as a panel title.
+        progress (bool):
+            Flag determining whether the progress of making the movie is shown.
     """
-    quantities = [{'title': 'Concentration', 'source': None, 'scale': scale}]
+    quantities = [{'title': 'Concentration', 'source': None}]
     if extras:
         for key, value in extras.items():
             quantities.append({'title': key, 'source': value})
     
     # initialize the plot with the first data point
-    plot = ScalarFieldPlot.from_storage(storage, quantities=quantities)
+    plot = ScalarFieldPlot.from_storage(storage, quantities=quantities,
+                                        scale=scale)
     # make the full movie
     plot.make_movie(storage, filename, progress=progress)    
    
     
     
+@fill_in_docstring
 def movie_multiple(storage: StorageBase,
                    filename: str,
                    quantities=None,
+                   scale: ScaleData = 'automatic',
                    progress: bool = True) -> None:
     """ produce a movie for a simulation with n components
 
     Args:
         storage (:class:`~pse.storage.base.StorageBase`):
             The storage instance that contains all the data for the movie
-        filename (str): The filename to which the movie is written. The
-            extension determines the format used.
-        quantities: |Args_plot_quantities| 
-        progress (bool): Flag determining whether the progress of making
-            the movie is shown.
+        filename (str):
+            The filename to which the movie is written. The extension determines
+            the format used.
+        quantities:
+            {ARG_PLOT_QUANTITIES}
+        scale (str, float, tuple of float):
+            {ARG_PLOT_SCALE}
+        progress (bool):
+            Flag determining whether the progress of making the movie is shown.
     """
-    plot = ScalarFieldPlot.from_storage(storage, quantities=quantities)
+    plot = ScalarFieldPlot.from_storage(storage, quantities=quantities,
+                                        scale=scale)
     plot.make_movie(storage, filename, progress=progress)
 
